--[[Author:		Cyprias 
	License:	All Rights Reserved
	Contact:	Cyprias on Curse.com or WowAce.com

	Only Curse.com and Wowace.com have permission to host this addon.
	I have not given permission for Plate Buffs to be used in any addon compilation or UI pack.]]
local folder, core = ...
	
-- global lookup
local _G = _G
local pairs = pairs
local InterfaceOptionsFrame_OpenToCategory = InterfaceOptionsFrame_OpenToCategory
local GetTime = GetTime
local table_sort = table.sort
local string_sub = string.sub
local string_len = string.len
local LibStub = LibStub
local table_getn = table.getn
local UnitGUID = UnitGUID
local UnitBuff = UnitBuff
local UnitDebuff = UnitDebuff
local UnitAura = UnitAura
local table_insert = table.insert
local UnitExists = UnitExists
local UnitIsUnit = UnitIsUnit
local Debug = core.Debug
local UnitIsPlayer = UnitIsPlayer
local UnitClassification = UnitClassification
local UnitName = UnitName

--~ PlateBuffs = core --global table

local LibNameplate = LibStub("LibNameplate-1.0", true)
if not LibNameplate then	error(folder .. " requires LibNameplate-1.0.") return end

-- Isolate the environment
--~ local _G = getfenv(0)
--~ setmetatable(core, {__index = _G})
--~ setfenv(1, core)


-- local
core.title		= "Plate Buffs"
core.version	= GetAddOnMetadata(folder, "X-Curse-Packaged-Version") or ""
core.titleFull	= core.title.." "..core.version
core.addonDir = "Interface\\AddOns\\PlateBuffs\\"

core.LibNameplate = LibNameplate

LibStub("AceAddon-3.0"):NewAddon(core, folder, "AceConsole-3.0", "AceEvent-3.0") 
--~ L = LibStub("AceLocale-3.0"):GetLocale(folder, true)

core.L = LibStub("AceLocale-3.0"):GetLocale(folder, true)
local L = core.L

local regEvents = {
	"PLAYER_TARGET_CHANGED",
	"UPDATE_MOUSEOVER_UNIT",
	"UNIT_AURA",
	"UNIT_TARGET",
}
core.db	= {}
local P	= {} --db.profile


core.defaultSettings	= {
	profile = {
	},
}

core.buffFrames = {}
core.guidBuffs = {}
core.nametoGUIDs = {}

local guidBuffs = core.guidBuffs
local nametoGUIDs = core.nametoGUIDs
local buffFrames = core.buffFrames
local defaultSettings = core.defaultSettings

core.iconTestMode = false
local coreOpts

function core:OnInitialize()
	local db = LibStub("AceDB-3.0"):New("PB_DB", core.defaultSettings)
	self.CoreOptionsTable.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(db)--save option profile or load another chars opts.
	db.RegisterCallback(self, "OnProfileChanged", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileCopied", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileReset", "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileDeleted", "OnProfileChanged")
	self:RegisterChatCommand("pb", "MySlashProcessorFunc")
	
	
	local config = LibStub("AceConfig-3.0")
	local dialog = LibStub("AceConfigDialog-3.0")
	config:RegisterOptionsTable(self.title, self.CoreOptionsTable)
	coreOpts = dialog:AddToBlizOptions(self.title, self.titleFull)
	
	self.db = db
end

function core:OnEnable()
	P = self.db.profile

	for i, event in pairs(regEvents) do 
		self:RegisterEvent(event)
	end
	
	LibNameplate.RegisterCallback(self, "LibNameplate_NewNameplate")
	LibNameplate.RegisterCallback(self, "LibNameplate_FoundGUID")
	LibNameplate.RegisterCallback(self, "LibNameplate_RecycleNameplate")
	LibNameplate.RegisterCallback(self, "LibNameplate_VisiblePlateChange")
	
	for plate in pairs(LibNameplate.nameplates) do 
		if plate:IsShown() then
			self:LibNameplate_NewNameplate("LibNameplate_NewNameplate", plate)
		end
	end
	
--~ 	Debug("OnEnable", "core", self.db.profile)
end

function core:OnDisable()
	LibNameplate.UnregisterAllCallbacks(self) 
	
	
	for plate in pairs(core.buffBars) do 
		for i=1, table_getn(core.buffBars[plate]) do 
			core.buffBars[plate][i]:Hide() --makesure all frames stop OnUpdating.
		end
	end
end

----------------------------------------------------------------------
function core:OnProfileChanged(...)									--
-- User has reset proflie, so we reset our spell exists options.	--
----------------------------------------------------------------------
	-- Shut down anything left from previous settings
	self:Disable()
	-- Enable again with the new settings
	self:Enable()
end

----------------------------------------------
function core:MySlashProcessorFunc(input)	--
-- /da function brings up the UI options.	--
----------------------------------------------
	InterfaceOptionsFrame_OpenToCategory(coreOpts)
end


function core:LibNameplate_VisiblePlateChange(event, plate, visiblePlate)
--~ 	Debug("VisiblePlateChange", plate, visiblePlate)
	if buffFrames[plate] then
		if buffFrames[plate][1] then
			self:ResetBarPoint(buffFrames[plate][1], plate, visiblePlate)
		end
		for i=2, table_getn(buffFrames[plate]) do 
			buffFrames[plate][i]:SetParent(visiblePlate)
		end
	end
end




function core:LibNameplate_RecycleNameplate(event, plate)
	if buffFrames[plate] then
		for i=1, table_getn(buffFrames[plate]) do 
			buffFrames[plate][i]:Hide()
		end
	end
--~ 	Debug("RecycleNameplate", LibNameplate:GetName(plate))
end



local function ShouldAddBuffs(plate)
	local plateType = LibNameplate:GetType(plate)
	local plateReaction = LibNameplate:GetReaction(plate)

	if (P.abovePlayers == true and plateType == "PLAYER") or (P.aboveNPC == true and plateType == "NPC") then
		if P.aboveFriendly == true and plateReaction == "FRIENDLY" then
			return true
		elseif P.aboveNeutral == true and plateReaction == "NEUTRAL" then
			return true
		elseif P.aboveHostile == true and plateReaction == "HOSTILE" then
			return true
		end
	end
	
	return false
end
	
function core:LibNameplate_NewNameplate(event, plate)
	
	local plateName =  LibNameplate:GetName(plate) or "UNKNOWN"
	if ShouldAddBuffs(plate) == true then
		if nametoGUIDs[plateName] and (LibNameplate:GetType(plate) == "PLAYER" or LibNameplate:IsBoss(plate)) then
			self:AddBuffsToPlate(plate, nametoGUIDs[plateName])
		elseif P.unknownSpellDataIcon == true then
			self:AddUnknownIcon(plate)
		end
	end

end

function core:LibNameplate_FoundGUID(event, plate, GUID, unitID)
	local plateName =  LibNameplate:GetName(plate)
--~ 	Debug("FoundGUID", plateName, plate:GetFrameLevel(), plate:GetFrameStrata())
--~ 	Debug("FoundGUID", LibNameplate:GetName(plate))
	

	--[[]

	Debug("FoundGUID", plateName, 
--~ 		plate,
		GUID, 
		unitID,
--~ 		LibNameplate:GetLevel(plate),
--~ 		LibNameplate:GetReaction(plate),
--~ 		LibNameplate:GetType(plate),
--~ 		LibNameplate:IsBoss(plate),
--~ 		LibNameplate:GetHealthMax(plate),
--~ 		LibNameplate:GetClass(plate),
--~ 		LibNameplate:IsElite(plate),
--~ 		LibNameplate:GetThreatSituation(plate),
--~ 		LibNameplate:GetAlpha(plate),
--~ 		LibNameplate:IsTarget(plate),
--~ 		LibNameplate:GetHealth(plate), 
--~ 		LibNameplate:GetHealthMax(plate),
--~ 		LibNameplate:IsMouseover(plate),
--~ 		LibNameplate:IsCasting(plate),
--~ 		LibNameplate:IsInCombat(plate),
--~ 		LibNameplate:IsMarked(plate),
--~ 		LibNameplate:GetRaidIcon(plate),
--~ 		LibNameplate:GetGUID(plate)
		"" --nada
	)
]]
	if ShouldAddBuffs(plate) == true then
		self:AddBuffsToPlate(plate, GUID)
	end
	
	--Do nothing values to test library.
	local value = LibNameplate:GetName(plate)
	value = LibNameplate:GetLevel(plate)
	value = LibNameplate:GetReaction(plate)
	value = LibNameplate:GetType(plate)
	value = LibNameplate:IsBoss(plate)
	value = LibNameplate:GetHealthMax(plate)
	value = LibNameplate:GetClass(plate)
	value = LibNameplate:IsElite(plate)
	value = LibNameplate:GetThreatSituation(plate)
	value = LibNameplate:GetAlpha(plate)
	value = LibNameplate:IsTarget(plate)
	value = LibNameplate:GetHealth(plate)
	value = LibNameplate:GetHealthMax(plate)
	value = LibNameplate:IsMouseover(plate)
	value = LibNameplate:IsCasting(plate)
	value = LibNameplate:IsInCombat(plate)
	value = LibNameplate:IsMarked(plate)
	value = LibNameplate:GetRaidIcon(plate)
	value = LibNameplate:GetGUID(plate)
--~ 	value = 

--~ 	Debug("LibNameplate_FoundGUID")
--~ 	Debug("GetName", LibNameplate:GetName(plate))
--~ 	Debug("GetLevel", LibNameplate:GetLevel(plate))
--~ 	Debug("GetReaction", LibNameplate:GetReaction(plate))
--~ 	Debug("GetType", LibNameplate:GetType(plate))
--~ 	Debug("IsBoss", LibNameplate:IsBoss(plate))
--~ 	Debug("GetHealthMax", LibNameplate:GetHealthMax(plate))
--~ 	Debug("GetClass", LibNameplate:GetClass(plate))
--~ 	Debug("IsElite", LibNameplate:IsElite(plate))
--~ 	Debug("GetThreatSituation", LibNameplate:GetThreatSituation(plate))
--~ 	Debug("GetAlpha", LibNameplate:GetAlpha(plate))
--~ 	Debug("IsTarget", LibNameplate:IsTarget(plate))
--~ 	Debug("GetHealth", LibNameplate:GetHealth(plate), "GetHealthMax", LibNameplate:GetHealthMax(plate))
--~ 	Debug("IsMouseover", LibNameplate:IsMouseover(plate))
--~ 	Debug("IsCasting", LibNameplate:IsCasting(plate))
--~ 	Debug("IsInCombat", LibNameplate:IsInCombat(plate))
--~ 	Debug("IsMarked", LibNameplate:IsMarked(plate))
--~ 	Debug("GetRaidIcon", LibNameplate:GetRaidIcon(plate))
--~ 	Debug("GetGUID", LibNameplate:GetGUID(plate))

	
end


local function CollectUnitInfo(unitID)
	local GUID = UnitGUID(unitID)
	
--~ 	core:Debug("CollectUnitInfo", unitID, GUID)
	guidBuffs[GUID] = {}
	
	
	
	local i = 1;
	if P.showBuffs == true then
		local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId
		while UnitBuff(unitID, i) do 
			name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId = UnitBuff(unitID, i)
			if P.defaultSpells == 1 or (P.defaultSpells == 2 and unitCaster and unitCaster == "player") then
				if core.iconTestMode == true then
					for t=1, P.iconsPerRow do 
						table_insert(guidBuffs[GUID], {
							icon = icon,
							expirationTime = expirationTime,
							startTime = expirationTime - duration,
							duration = duration,
							playerCast = unitCaster and unitCaster == "player" and 1,
							stackCount = count,
						})
					end
				else
					table_insert(guidBuffs[GUID], {
						icon = icon,
						expirationTime = expirationTime,
						startTime = expirationTime - duration,
						duration = duration,
						playerCast = unitCaster and unitCaster == "player" and 1,
						stackCount = count,
					})
				end
			end
			i=i + 1
		end
	end
	
	if P.showDefuffs == true then
	i = 1;
		local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId
		while UnitDebuff(unitID, i) do 
			name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId = UnitDebuff(unitID, i) 
			if P.defaultSpells == 1 or (P.defaultSpells == 2 and unitCaster and unitCaster == "player") then
				if core.iconTestMode == true then
					for t=1, P.iconsPerRow do 
						table_insert(guidBuffs[GUID], {
							icon = icon,
							expirationTime = expirationTime,
							startTime = expirationTime - duration,
							duration = duration,
							playerCast = unitCaster and unitCaster == "player" and 1,
							stackCount = count,
						})
					end
				else
					table_insert(guidBuffs[GUID], {
						icon = icon,
						expirationTime = expirationTime,
						startTime = expirationTime - duration,
						duration = duration,
						playerCast = unitCaster and unitCaster == "player" and 1,
						stackCount = count,
					})
				end
				
			end
			i=i + 1
		end
	end
	
	if P.showAura == true then
		i = 1;
		local name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId
		while UnitAura(unitID, i) do 
			name, rank, icon, count, debuffType, duration, expirationTime, unitCaster, isStealable, shouldConsolidate, spellId = UnitAura(unitID, i) 
			if P.defaultSpells == 1 or (P.defaultSpells == 2 and unitCaster and unitCaster == "player") then
				if core.iconTestMode == true then
					for t=1, P.iconsPerRow do 
						table_insert(guidBuffs[GUID], {
							icon = icon,
							expirationTime = expirationTime,
							startTime = expirationTime - duration,
							duration = duration,
							playerCast = unitCaster and unitCaster == "player" and 1,
							stackCount = count,
						})
					end
				else
					table_insert(guidBuffs[GUID], {
						icon = icon,
						expirationTime = expirationTime,
						startTime = expirationTime - duration,
						duration = duration,
						playerCast = unitCaster and unitCaster == "player" and 1,
						stackCount = count,
					})
				end
			end
			i=i + 1
		end
	end
	
	
	
	if UnitIsPlayer(unitID) or UnitClassification(unitID) == "worldboss" then
		--These should always have unique names.
		nametoGUIDs[UnitName(unitID)] = GUID
	end
	
	--nametoGUIDs
	
	local plate = LibNameplate:GetNameplateByGUID(GUID)
	if plate and ShouldAddBuffs(plate) == true then
		core:AddBuffsToPlate(plate, GUID)
	end

	
end


function core:PLAYER_TARGET_CHANGED(event, ...)
	if UnitExists("target") then
		CollectUnitInfo("target")
	end
end


function core:UNIT_TARGET(event, unitID)--this fires half a second after PLAYER_TARGET_CHANGED. Not good for rapid targeting.
	if not UnitIsUnit(unitID, "player") and UnitExists(unitID.."target") then
		CollectUnitInfo(unitID.."target")
	end
--~ 	Debug(event, ...)
end


function core:UPDATE_MOUSEOVER_UNIT(event, ...)
	if UnitExists("mouseover") then
		CollectUnitInfo("mouseover")
	end
--~ 	Debug(event, ...)
end

function core:UNIT_AURA(event, unitID)
	if UnitExists(unitID) then
		CollectUnitInfo(unitID)
	end
--~ 	Debug(event, ...)
end
